// bmptoc.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "bmptoc.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// The one and only application object

CWinApp theApp;
using namespace std;
CString csICatalog = "";	//input  catalog
CString csOCatalog = "";	//output catalog
CString csIFile = "";		// input filename (full)
CString csOFile = "";		//output filename (full)
CString csMakeRawBmpDump = "";
CString csMakeBinaryDump = "";

void Usage (void)
{
	CString csExe;
	csExe = theApp.m_pszExeName;

	printf("Simple BMP to dump converter utility.\n");
	printf("Usage: %s.exe filename[.bmp] [outputfilename] [-raw] [-noheader]\n", csExe);
	printf("If outputfilename omitted, dump filename[.bmp] to console.\n");
	printf("-raw option mean no cut additional line aligment bytes.\n");
	printf("-noheader simply convert source as binary file.\n");
	exit (1);
}

void ParseCommandLine (int argc, TCHAR* argv[])
{
	CString argument;
	CString testarg;
	int iArgIdx;

	if (argc < 2)
		Usage();
	for (iArgIdx=1; iArgIdx < argc; iArgIdx++)
	{
		argument = argv[iArgIdx];
		argument = argument.Trim();
		testarg  = argument;
		if ("-raw" == testarg.MakeLower())
		{
			csMakeRawBmpDump = testarg;
			continue;
		}
		if ("-noheader" == testarg.MakeLower())
		{
			csMakeBinaryDump = testarg;
			continue;
		}
		if (!csOFile.IsEmpty())
			continue;
		if (!csIFile.IsEmpty())
		{
			csOFile = argument;
			continue;
		}
		csIFile = argument;
	}
}

CString CreateCharComment (CString charname)
{
	CString csSample = "char34";
	UCHAR CharVal;
	char Sym1, Sym2;
	CString comment;

	charname = charname.Trim();
	if (charname.Left(4) != csSample.Left(4))
		return "";
	if (charname.GetLength() != csSample.GetLength())
		return "";
	//create char comment
	CString csHexStr = charname.Right(2).MakeUpper();
	Sym1 = *csHexStr.GetBuffer();
	Sym2 = *(csHexStr.GetBuffer()+1);
	switch (Sym1)
	{
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
		CharVal = (Sym1 - '0') * 16;
		break;
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
		CharVal = (Sym1 - 'A' + 10) * 16;
		break;
	}
	switch (Sym2)
	{
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
		CharVal += Sym2 - '0';
		break;
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
		CharVal += (Sym2 - 'A' + 10);
		break;
	}
	switch (CharVal)
	{
	case 0:
		comment.Format("('zero')");
		break;
	case 0x0A:
		comment.Format("('LF')");
		break;
	case 0x0D:
		comment.Format("('CR')");
		break;
	case 0x19:
		comment.Format("('end of medium')");
		break;
	case 0x1A:
		comment.Format("('substitute')");
		break;
	default:
		comment.Format("('%c')", (char)CharVal);
	}
	return comment;
}

void DumpBmpToC (CString csIFile, CString csOFile, CString csArray)
//Convert input file csIFile (uncompressed BMP) to text C-dump csOFile,
// comment created from array name csArray.
//About BMP format header see http://pascal.sources.ru/articles/099.htm
{
	//step 1. Get parameters X and Y size of image data.
	CBitmap bitmap;
	BITMAP bmp;
	HBITMAP hBitmap;

	hBitmap = (HBITMAP)::LoadImage(AfxGetInstanceHandle(), csIFile, 
		                           IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
	if(hBitmap) 
	{
		if(bitmap.DeleteObject()) 
			bitmap.Detach();
		bitmap.Attach(hBitmap);
		bitmap.GetBitmap(&bmp);
		//bmp.bmWidth  == Y size
		//bmp.bmHeight == X size
	}
	else 
	{
		printf("Cannot load bitmap...\n");
		exit(1);
	}

	//step 2. Read RAW file data.
	FILE *fBinary;
	FILE *fText;
	UCHAR buf[256];
	UCHAR bOneByte;
	size_t dwFileSize, dwFileReaded;
	size_t dwBmpSize, dwBmpReaded;
	size_t dwOffsetBits, dwSkip;
	CString csBmp;
	int iLineCnt;
	long uLineLength;	//    

	fBinary = fopen(csIFile, "rb");
	if (NULL == fBinary)
	{
		printf("Cannot open %s binary input file.\n", csIFile);
		Usage();
	}
	dwFileReaded = 0;
	//skip "BM" signature
	dwFileReaded += fread(buf, 1, 2 /* 2 bytes read */, fBinary);
	//read file size
	dwFileReaded += fread(&dwFileSize, 1, 4 /* 4 bytes read */, fBinary);
	//skip Reserved 1 and Reserved 2
	dwFileReaded += fread(buf, 1, 2 /* 2 bytes read */, fBinary);
	dwFileReaded += fread(buf, 1, 2 /* 2 bytes read */, fBinary);
	//read offset image data
	dwFileReaded += fread(&dwOffsetBits, 1, 4 /* 4 bytes read */, fBinary);
	//skip remain header data
	dwSkip = dwOffsetBits - dwFileReaded;
	dwFileReaded += fread(buf, 1, dwSkip /* dwSkip bytes read */, fBinary);
	csBmp = "/* " + csArray + " " + CreateCharComment(csArray) + " */\n";
	iLineCnt = 0;
	//*** start create dump text ***
	//calculate USEFUL line size in bytes
	uLineLength = bmp.bmWidth / 8;
	if ((uLineLength * 8) < bmp.bmWidth)
		uLineLength++;
	//calculate REAL line size in bytes (align to margin 4 byte)
	long uWhole, uFrac, uRealLineLength;
	uWhole = uLineLength / 4;
	uFrac  = uLineLength % 4;
	uRealLineLength = uWhole * 4;
	if (0 != uFrac)
		uRealLineLength += 4;
	long uLineByteCnt = 0;
    dwBmpSize = bmp.bmHeight*uLineLength;
    dwBmpReaded = 0;
	while ((dwFileReaded < dwFileSize)&&(dwBmpReaded < dwBmpSize))
	{
		dwFileReaded += fread(&bOneByte, 1, 1, fBinary);
        if (uLineByteCnt < uLineLength)
        {
            // , 
            sprintf((char*)buf, "0x%.2X", bOneByte);
		    csBmp += CString(buf);
            dwBmpReaded++;
		    if ((dwFileReaded < dwFileSize)&&(dwBmpReaded < dwBmpSize))
		    {
		        csBmp += ",";
			    if (iLineCnt<7)
				    iLineCnt++;
			    else
			    {
				    iLineCnt = 0;
				    csBmp += "\n";
			    }
		    }
        }
        uLineByteCnt++;
        if (uLineByteCnt>=uRealLineLength)
            uLineByteCnt=0;
	}
	csBmp += "\n";
	//*** end create dump text ***
	fclose(fBinary);
	if (!csOFile.IsEmpty())
	{
		fText = fopen(csOFile, "wt");
		if (NULL == fText)
		{
			printf("Cannot open %s text output file.\n", csOFile);
			Usage();
		}
		fwrite(csBmp.GetBuffer(),1,csBmp.GetLength(), fText);
		fclose(fText);
	}
	else
		printf(csBmp);
}

void DumpBmpToCRaw (CString csIFile, CString csOFile, CString csArray)
//This version not cut unused line bytes
{
	FILE *fBinary;
	FILE *fText;
	UCHAR buf[256];
	UCHAR bOneByte;
	size_t dwFileSize, dwReaded;
	size_t dwOffsetBits, dwSkip;
	CString csBmp;
	int iLineCnt;

	fBinary = fopen(csIFile, "rb");
	if (NULL == fBinary)
	{
		printf("Cannot open %s binary input file.\n", csIFile);
		Usage();
	}
	dwReaded = 0;
	//skip "BM" signature
	dwReaded += fread(buf, 1, 2 /* 2 bytes read */, fBinary);
	//read file size
	dwReaded += fread(&dwFileSize, 1, 4 /* 4 bytes read */, fBinary);
	//skip Reserved 1 and Reserved 2
	dwReaded += fread(buf, 1, 2 /* 2 bytes read */, fBinary);
	dwReaded += fread(buf, 1, 2 /* 2 bytes read */, fBinary);
	//read offset image data
	dwReaded += fread(&dwOffsetBits, 1, 4 /* 4 bytes read */, fBinary);
	//skip remain header data
	dwSkip = dwOffsetBits - dwReaded;
	dwReaded += fread(buf, 1, dwSkip /* dwSkip bytes read */, fBinary);
	csBmp = "/* " + csArray + " " + CreateCharComment(csArray) + " */\n";
	iLineCnt = 0;
	//*** start create dump text ***
	while (dwReaded < dwFileSize)
	{
		dwReaded += fread(&bOneByte, 1, 1, fBinary);
		sprintf((char*)buf, "0x%.2X", bOneByte);
		csBmp += CString(buf);
		csBmp += ",";
		if (dwReaded < dwFileSize)
		{
			if (iLineCnt<7)
				iLineCnt++;
			else
			{
				iLineCnt = 0;
				csBmp += "\n";
			}
		}
	}
	csBmp += "\n";
	//*** end create dump text ***
	fclose(fBinary);
	if (!csOFile.IsEmpty())
	{
		fText = fopen(csOFile, "wt");
		if (NULL == fText)
		{
			printf("Cannot open %s text output file.\n", csOFile);
			Usage();
		}
		fwrite(csBmp.GetBuffer(),1,csBmp.GetLength(), fText);
		fclose(fText);
	}
	else
		printf(csBmp);
}

void DumpBinToC (CString csIFile, CString csOFile, CString csArray)
//Convert input file csIFile 'as is'
{
	FILE *fBinary;
	FILE *fText;
	UCHAR buf[256];
	UCHAR bOneByte;
	size_t dwReaded;
	CString csBmp;
	int iLineCnt;

	fBinary = fopen(csIFile, "rb");
	if (NULL == fBinary)
	{
		printf("Cannot open %s binary input file.\n", csIFile);
		Usage();
	}
	dwReaded = 0;
	iLineCnt = 0;
	//*** start create dump text ***
	while (1 == fread(&bOneByte, 1, 1, fBinary))
	{
		dwReaded++;
		sprintf((char*)buf, "0x%.2X", bOneByte);
		csBmp += CString(buf);
		csBmp += ",";
        if (iLineCnt<7)
			iLineCnt++;
		else
		{
			iLineCnt = 0;
			csBmp += "\n";
		}
	}
	csBmp += "\n";
	//*** end create dump text ***
	fclose(fBinary);
	if (!csOFile.IsEmpty())
	{
		fText = fopen(csOFile, "wt");
		if (NULL == fText)
		{
			printf("Cannot open %s text output file.\n", csOFile);
			Usage();
		}
		fwrite(csBmp.GetBuffer(),1,csBmp.GetLength(), fText);
		fclose(fText);
	}
	else
		printf(csBmp);
}

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;
	CString csDrive, csDir, csName, csExt;
	char drive[3];
	char dir[256];
	char fname[256];
	char ext[256];

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		_tprintf(_T("Fatal Error: MFC initialization failed\n"));
		nRetCode = 1;
	}
	else
	{
		// TODO: code your application's behavior here.
		ParseCommandLine(argc, argv);
		//     
		_splitpath(csIFile, drive, dir, fname, ext);
		csDrive = (CString)drive;
		csDir   = (CString)dir;
		csName  = (CString)fname;
		csExt   = (CString)ext;
		if (csDrive != "")
			csDir  = csDrive + csDir;
        if (csMakeBinaryDump == "-noheader")
            DumpBinToC    (csIFile, csOFile, csName);  
		else if (csMakeRawBmpDump != "-raw")
			DumpBmpToC    (csIFile, csOFile, csName);
		else
			DumpBmpToCRaw (csIFile, csOFile, csName);
	}

	return nRetCode;
}
